'                       Simulation_A
'
' This object provides a simulated Stamp for Sid's Propeller system.
' It makes use of the way the I/O pins are connected on the Propeller.
' In particular, this routine can wait for another cog to send out
' serial data, then respond to it as if it were an external processor.
' The serial routines are taken from the BS2 compatibility object.

' Call using start(<output>,<input>,<Baud>,<mode>).
'  <output>  is the pin # used as an output by the rest of the program.
'  <input>   is the pin # used as an input by the rest of the program.
'  <Baud>    is the Baud used (maximum 9600)
'  <mode>    is either Inv or NInv (see below)
'
'  Modified 8/1/07 - Added sendhex and senddec

CON
  Inv        = 0        ' Inverted - Normally low
  NInv       = 1        ' Non-Inverted - Normally high

VAR
  long myStack[64]', RFIDstack[64]
  
PUB startStamp(inpin, outpin, Baud, mode) : okay
  outa[inpin] := mode
  dira[inpin] := 1     ' Make sure I/O pins set up properly
  dira[outpin] := 0    ' (Opposite to what's set in the cog)
  okay := cognew(Simulate(inpin,outpin,Baud,mode),@myStack) <> -1

PUB startRfid(enable, outpin, Baud, mode) : okay
  outa[enable] := 1    ' enable is always low true
  dira[enable] := 1    ' Make sure I/O pins set up properly
  dira[outpin] := 0    ' (Opposite to what's set in the cog)
  'okay := cognew(DoRFID(enable,outpin,Baud,mode),@RFIDstack) <> -1

PRI Simulate(inpin, outpin, Baud, mode) | c
  outa[outpin] := mode                                ' Setup I/O and wait for
  dira[outpin] := 1                                   '  things to stabilize
  dira[inpin] := 0
  waitcnt(clkfreq / 10 + cnt)
  repeat
    c := serialIn(inpin,Baud,mode,8)
    waitcnt(clkfreq / 100 + cnt)                      ' Wait 10ms before responding
    case c
      0:                                              ' Ask Stamp to reinitialize
        serialStr(outpin,string("Init",10),Baud,mode,8)
      "G":
        serialStr(outpin,string("235959",10),Baud,mode,8)
      other:                                          ' A single letter command
        serialStr(outpin,string("Cmd:"),Baud,mode,8)
        serialOut(outpin,c,Baud,mode,8)
        serialOut(outpin,10,Baud,mode,8)

PRI DoRFID(enable, outpin, Baud, mode) | id
  id := 0
  outa[outpin] := mode                                ' Setup I/O and wait for
  dira[outpin] := 1                                   '  things to stabilize
  dira[enable] := 0
  waitcnt(clkfreq / 10 + cnt)
  repeat
    repeat until ina[enable]                          ' Wait for idle enable
    repeat while ina[enable]                          ' Wait for active enable
    waitcnt(clkfreq / 100 + cnt)                      ' Wait 10ms before responding
    serialStr(outpin,string("UVWXYZ"),Baud,mode,8)
    case id
      0: serialStr(outpin,string("46CDD"),Baud,mode,8)
      1: serialStr(outpin,string("46CB2"),Baud,mode,8)
      2: serialStr(outpin,string("00000"),Baud,mode,8)
    serialOut(outpin,13,Baud,mode,8)
    id := (id + 1) // 3
  
PUB serialIn(pin, Baud, mode, bits) : byteVal | bitTime, time
{{ Accepts asynchronous byte value on defined pin, at Baud, in mode for #bits
   Mode: 0 = Inverted - Normally low        Constant: Inv
         1 = Non-Inverted - Normally high   Constant: NInv }}
  dira[pin]~                                          ' Set up input pin as an input
  bitTime := clkfreq / Baud                           ' Calculate bit time for Baud
  waitpeq(mode << pin, |< pin, 0)                     ' Wait for idle (should be idle)
  waitpne(mode << pin, |< pin, 0)                     ' Now wait for start of start bit
  time := cnt                                         ' Wait until middle of start bit
  waitcnt(time += bitTime >> 1)
  repeat bits                                         ' Sample input at the middle of
    waitcnt(time += bitTime)                          '  each data bit time for #bits
    byteVal := ((ina[pin] ^ (1 - mode)) << bits | byteVal) >> 1
     
PUB serialOut(pin, char, Baud, mode, bits ) | bitTime, time
{{ Send asynchronous character (byte) on defined pin, at Baud, in Mode for #bits
   Mode: 0 = Inverted - Normally low        Constant: Inv
         1 = Non-Inverted - Normally high   Constant: NInv }}
  outa[pin] := mode                                   ' Set up output pin with the proper
  dira[pin]~~                                         '  idle state and as an output
  bitTime := clkfreq / Baud                           ' Calculate bit time for Baud
  char := ((1 << bits | char) << 1) ^ (mode == 0)     ' Add start/stop bits & invert if needed
  time := cnt
  repeat bits + 2                                     ' Send each bit based on baud rate
    outa[Pin] := char
    char >>= 1
    waitcnt(time += bitTime)
  waitcnt(time += bitTime * (bits + 2))               ' Allow a character time for pacing

PUB serialStr(pin, str, Baud, mode, bits ) | c        ' Transmit a zero-terminated string
  repeat while (c := byte[str++]) <> 0
    serialOut(pin,c,Baud,mode,bits)

PUB serialHex(pin, value, digits, Baud, mode, bits)   ' Transmit a hexadecimal number
  value <<= (8 - digits) << 2                         ' Position the value based on # digits
  repeat digits                                       ' Rotate and translate to hex chars
    serialOut(pin,lookupz((value<-=4)&$F:"0".."9","A".."F"),Baud,mode,bits)